CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                            Pio (Fortran)                            C
C                                                                     C
C                    Polyglot Input/Output Library                    C
C                                                                     C
C               EPI Pomdapi, INRIA Paris - Rocquencourt               C
C                                                                     C
C  Copyright 2005-2014 INRIA.                                         C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C $Id: print_utils.fpp,v 1.2 2014/01/10 19:22:10 fclement Exp $

C     
C     Primitives for printing into the printing buffer.
C     
      
#include "pio_constants.h"
      
      subroutine buffer_print_char(sbr, c)
C     Print the character, ie 1-character long string, c.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), c
      
C     Local variable.
      integer loc
      
C     External functions.
      logical pio_get_debug_mode
      integer get_buffer_ptr
      
      loc = get_buffer_ptr(PIO_OUT)
      call set_buffer_char(sbr, PIO_OUT, loc, c)
      call incr_buffer_lim(sbr, PIO_OUT)
      call incr_buffer_ptr(sbr, PIO_OUT)
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ', ',
     +        'character |', c, '| printed in location ', loc
      endif
      
      return
      end
      
      
      subroutine buffer_print_printable_string(sbr, len, s)
C     Print the printable string s of length len.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), s*(*)
      integer len
      
C     Local variable.
      integer i
      
      do i = 1, len
         call buffer_print_char(sbr, s(i:i))
      enddo
      
      return
      end
      
      
      subroutine buffer_print_string(sbr, len, s)
C     Print the string s of length len.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), s*(*)
      integer len
      
C     Local variables.
      logical non_printable
      character backslash, c, s3*3
      integer i, d
      
C     External function.
      character backslash_char
      integer char_to_int
      
      backslash = backslash_char()
      
      do i = 1, len
         
         c = s(i:i)
C     Some compilers may complain, but there is no precision loss here
C     since d should be an integer less than 256.
         d = char_to_int(c)
         
         non_printable =
     +        d .le. 31 .or. d .eq. 34 .or. d .eq. 92 .or. d .ge. 127
         
         if (non_printable) then
            write(s3, '(i3.3)') d
            call buffer_print_char(sbr, backslash)
            call buffer_print_printable_string(sbr, 3, s3)
         else
            call buffer_print_char(sbr, c)
         endif
         
      enddo
      
      return
      end
      
      
      subroutine buffer_print_space(sbr)
C     Print a space character.
      
      implicit none
      
C     Input argument.
      character sbr*(*)
      
      call buffer_print_char(sbr, ' ')
      
      return
      end
      
      
      subroutine buffer_print_newline(sbr)
C     Print the end_of_line marker.
      
      implicit none
      
C     Input argument.
      character sbr*(*)
      
C     Local variable.
      character c
      
C     External function.
      character int_to_char
      
C     Newline character.
      c = int_to_char(10)
      
      call buffer_print_char(sbr, c)
      
      return
      end
      
      
      subroutine buffer_print_delimited_string(sbr, s, len)
C     Print a delimited string of characters of length len.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), s*(*)
      integer len
      
      call buffer_print_char(sbr, '"')
      call buffer_print_string(sbr, len, s)
      call buffer_print_char(sbr, '"')
      
      return
      end
      
      
      subroutine buffer_print_natural(sbr, n)
C     Print the natural number n as a string of decimal digits.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer n
      
C     Local variables.
      character sn*MAX_INT_LENGTH
      integer len
      
C     External function.
      logical pio_get_debug_mode
      
      call string_of_int(sbr, n, len, sn)
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ', ',
     +        'sn = |', sn(1:len), '|'
      endif
      
      call buffer_print_string(sbr, len, sn)
      
      return
      end
      
      
      subroutine buffer_print_int(sbr, n)
C     Print the integer number n.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer n
      
      if (n .lt. 0) then
         call buffer_print_char(sbr, '-')
      endif
      call buffer_print_natural(sbr, abs(n))
      
      return
      end
      
      
      subroutine buffer_print_natural_list2(sbr, n, lst)
C     Write the list of n naturals lst (for n > 1).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer n, lst(MAX_ARRAY_DIMENSION)
      
C     Local variables.
      integer i
      
      if (n .le. 1) then
         call error_too_short_list(sbr, 2, n)
      endif
      
      call buffer_print_natural(sbr, lst(1))
      do i = 2, n
         call buffer_print_string(sbr, 2, ', ')
         call buffer_print_natural(sbr, lst(i))
      enddo
      
      return
      end
      
      
      subroutine buffer_print_ascii_float(sbr, f)
C     Print the float number f in ASCII mode.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      FLOAT f
      
C     Local variable
      character sf*MAX_FLOAT_LENGTH
      integer len
      
C     External function.
      logical pio_get_debug_mode
      
      call string_of_float(f, len, sf)
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ', ',
     +        'sf = |', sf(1:len), '|'
      endif
      
      call buffer_print_string(sbr, len, sf)
      
      return
      end
      
      
      subroutine buffer_print_binary_float(sbr, f)
C     Print the float number f in binary mode.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      FLOAT f
      
C     Local variables.
      integer loc
      
C     External functions.
      character float_to_string_bin*FLOAT_SIZE
      integer get_buffer_ptr
      
      call buffer_print_char(sbr, '&')
      call buffer_print_size(sbr, FLOAT_SIZE)
      
      loc = get_buffer_ptr(PIO_OUT)
      
      call set_buffer_string
     +     (sbr, PIO_OUT, loc, FLOAT_SIZE, float_to_string_bin(f))
      
      call advance_buffer_lim(sbr, PIO_OUT, FLOAT_SIZE)
      call advance_buffer_ptr(sbr, PIO_OUT, FLOAT_SIZE)
      
      return
      end
